// ========================================================================
// Copyright 2007 Mort Bay Consulting Pty. Ltd.
// ------------------------------------------------------------------------
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at 
// http://www.apache.org/licenses/LICENSE-2.0
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//========================================================================

package dojox.cometd.demo;


import javax.servlet.ServletContextAttributeEvent;
import javax.servlet.ServletContextAttributeListener;

import dojox.cometd.Bayeux;
import dojox.cometd.Client;
import dojox.cometd.Listener;

public class BayeuxStartupListener implements ServletContextAttributeListener
{
    public void initialize(Bayeux bayeux)
    {
        synchronized(bayeux)
        {
            
            if(!bayeux.hasChannel("/rpc/echo"))
            {
                bayeux.subscribe("/rpc/echo",bayeux.newClient("echo",new EchoRPC(bayeux)));

                Client monitor_client=bayeux.newClient("monitor", new dojox.cometd.Listener()
                {
                    public void removed(String id,boolean timeout){}
                    public void deliver(Client fromClient, String toChannel, Object data,String id)
                    {
                        System.out.println("BayeuxStartupListener: "+fromClient+"-->"+toChannel+" "+data);
                    }
                });

                bayeux.subscribe("/meta/handshake",monitor_client);
                bayeux.subscribe("/meta/connect",monitor_client);
                bayeux.subscribe("/meta/subscribe",monitor_client);
                bayeux.subscribe("/meta/unsubscribe",monitor_client);
            }
        }
    }
    
    public void attributeAdded(ServletContextAttributeEvent scab)
    {
        if (scab.getName().equals(Bayeux.DOJOX_COMETD_BAYEUX))
        {
            Bayeux bayeux=(Bayeux)scab.getValue();
            initialize(bayeux);
        }
    }

    public void attributeRemoved(ServletContextAttributeEvent scab)
    {

    }

    public void attributeReplaced(ServletContextAttributeEvent scab)
    {

    }

    
    private static class EchoRPC implements Listener
    {
        Bayeux _bayeux;
        
        public EchoRPC(Bayeux bayeux)
        {
            _bayeux=bayeux;
        }

        public void removed(String id,boolean timeout){}
        
        public void deliver(Client fromClient, String toChannel, Object data, String msgId)
        {
            synchronized(this)
            {
                String reply_channel_id=toChannel+"/"+fromClient.getId();
                _bayeux.publish(fromClient,reply_channel_id,data,"R"+msgId);
            }
        }
        
    }
    
}
